@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::AnyThing;
-$Config::Model::AnyThing::VERSION = '2.097';
+$Config::Model::AnyThing::VERSION = '2.106';
 use Mouse;
 
 # FIXME: must cleanup warp mechanism to implement this
@@ -92,9 +92,9 @@ sub notify_change {
 
     return if $self->instance->initial_load and not $args{really};
 
-    $change_logger->debug( "called for ", $self->name, " from ", join( ' ', caller ),
+    $change_logger->trace( "called for ", $self->name, " from ", join( ' ', caller ),
         " with ", join( ' ', %args ) )
-        if $change_logger->is_debug;
+        if $change_logger->is_trace;
 
     # needs_save may be overridden by caller
     $args{needs_save} //= 1;
@@ -176,7 +176,7 @@ sub shorten_idx {
 sub xpath {
     my $self = shift;
 
-    $logger->debug("xpath called on $self");
+    $logger->trace("xpath called on $self");
 
     my $element = $self->element_name;
     $element = '' unless defined $element;
@@ -228,7 +228,7 @@ sub load_pod_annotation {
             $path =~ s/^[\s\*]+//;
             my $note = $item->text . '';
             $note =~ s/\s+$//;
-            $logger->debug("load_pod_annotation: '$path' -> '$note'");
+            $logger->trace("load_pod_annotation: '$path' -> '$note'");
             $self->grab( steps => $path )->annotation($note);
         }
     }
@@ -237,7 +237,7 @@ sub load_pod_annotation {
 # fallback method for object that don't implement has_data
 sub has_data {
     my $self= shift;
-    $logger->debug("called fall-back has_data for element", $self->name) if $logger->is_debug;
+    $logger->trace("called fall-back has_data for element", $self->name) if $logger->is_trace;
     return 1;
 }
 
@@ -274,13 +274,13 @@ sub _check_check {
 
 sub has_fixes {
     my $self = shift;
-    $logger->debug( "dummy has_fixes called on " . $self->name );
+    $logger->trace( "dummy has_fixes called on " . $self->name );
     return 0;
 }
 
 sub has_warning {
     my $self = shift;
-    $logger->debug( "dummy has_warning called on " . $self->name );
+    $logger->trace( "dummy has_warning called on " . $self->name );
     return 0;
 }
 
@@ -326,7 +326,7 @@ Config::Model::AnyThing - Base class for configuration tree item
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -350,7 +350,7 @@ Returns the element name that contain this object.
 For object stored in an array or hash element, returns the index (or key)
 containing this object.
 
-=head2 parent()
+=head2 parent
 
 Returns the node containing this object. May return undef if C<parent()> 
 is called on the root of the tree.
@@ -513,7 +513,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
