@@ -1,15 +1,17 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::BackendMgr;
-$Config::Model::BackendMgr::VERSION = '2.097';
+$Config::Model::BackendMgr::VERSION = '2.106';
 use Mouse;
+use strict;
+use warnings;
 
 use Carp;
 use 5.10.1;
@@ -23,6 +25,9 @@ use IO::File;
 use Storable qw/dclone/;
 use Scalar::Util qw/weaken/;
 use Log::Log4perl qw(get_logger :levels);
+use Path::Tiny 0.070;
+
+with "Config::Model::Role::ComputeFunction";
 
 my $logger = get_logger('BackendMgr');
 
@@ -71,6 +76,7 @@ sub get_tuned_config_dir {
     return $dir;
 }
 
+# check if dir is present. May create it in auto_create write mode
 sub get_cfg_dir_path {
     my $self = shift;
     my %args = @_;
@@ -90,11 +96,13 @@ sub get_cfg_dir_path {
         return ( 0, $dir );
     }
 
-    $logger->debug( "dir: " . $dir // '<undef>' );
+    $logger->trace( "dir: " . $dir // '<undef>' );
 
     return ( 1, $dir );
 }
 
+# return (1, config file path) constructed from arguments or return
+# (0). May create directory in auto_create write mode.
 sub get_cfg_file_path {
     my $self = shift;
     my %args = @_;
@@ -120,13 +128,12 @@ sub get_cfg_file_path {
     Config::Model::Exception::Model->throw(
         error  => "backend error: empty 'config_dir' parameter (and no config_file override)",
         object => $self->node
-    ) unless $args{config_dir} or $self->config_dir;
+    ) unless defined $args{config_dir} or defined $self->config_dir;
 
     my ( $dir_ok, $dir ) = $self->get_cfg_dir_path(%args);
 
     if ( defined $args{file} ) {
-        my $file = $args{file};
-        $file =~ s/&index/$self->node->index_value/eg;
+        my $file = $args{skip_compute} ? $args{file} : $self->node->compute_string($args{file});
         my $res = $dir . $file;
         $logger->trace("get_cfg_file_path: returns $res");
         return ( $dir_ok, $res );
@@ -137,6 +144,7 @@ sub get_cfg_file_path {
         return (0);
     }
 
+    # constructing config file name with >instance name>.<suffix>
     my $i    = $self->node->instance;
     my $name = $dir . $i->name;
 
@@ -161,28 +169,20 @@ sub get_cfg_file_path {
 }
 
 sub open_read_file {
-    my $self = shift;
-    my %args = @_;
+    my ($self, $backend, $file_path) = @_;
 
-    my ( $file_ok, $file_path ) = $self->get_cfg_file_path(%args);
-
-    if ( $file_ok and $file_path eq '-' ) {
+    if ( $file_path eq '-' ) {
         my $io = IO::Handle->new();
         if ( $io->fdopen( fileno(STDIN), "r" ) ) {
-            return ( 1, '-', $io );
+            return $io;
         }
         else {
-            return ( 0, '-' );
+            return;
         }
     }
 
-    # not very clean
-    return ( 0, $file_path )
-        if $args{backend} =~ /_file$/
-        and ( not $file_ok or not -r $file_path );
-
     my $fh = new IO::File;
-    if ( $file_ok and -e $file_path ) {
+    if ( -e $file_path ) {
         $logger->debug("open_read_file: open $file_path for read");
         $fh->open($file_path);
         $fh->binmode(":utf8");
@@ -190,10 +190,10 @@ sub open_read_file {
         # store a backup in memory in case there's a problem
         $self->file_backup( [ $fh->getlines ] );
         $fh->seek( 0, 0 );    # go back to beginning of file
-        return ( 1, $file_path, $fh );
+        return $fh;
     }
     else {
-        return ( 0, $file_path );
+        return;
     }
 }
 
@@ -205,7 +205,7 @@ sub load_backend_class {
     my $backend  = shift;
     my $function = shift;
 
-    $logger->debug("load_backend_class: called with backend $backend, function $function");
+    $logger->trace("load_backend_class: called with backend $backend, function $function");
     my %c;
 
     my $k = "Config::Model::Backend::" . ucfirst($backend);
@@ -230,7 +230,7 @@ sub load_backend_class {
     # look for file to load
     my $class_to_load;
     foreach my $c ( keys %c ) {
-        $logger->debug("load_backend_class: looking to load class $c");
+        $logger->trace("load_backend_class: looking to load class $c");
         foreach my $prefix (@INC) {
             my $realfilename = "$prefix/$c{$c}";
             $class_to_load = $c if -f $realfilename;
@@ -240,7 +240,7 @@ sub load_backend_class {
     return unless defined $class_to_load;
     my $file_to_load = $c{$class_to_load};
 
-    $logger->debug("load_backend_class: loading class $class_to_load, $file_to_load");
+    $logger->trace("load_backend_class: loading class $class_to_load, $file_to_load");
     eval { require $file_to_load; };
 
     if ($@) {
@@ -252,10 +252,7 @@ sub load_backend_class {
 sub read_config_data {
     my ( $self, %args ) = @_;
 
-    {
-        no warnings 'uninitialized';
-        $logger->debug( "called with ", join( ' ', %args ) );
-    }
+    $logger->trace( "called for node ", $self->node->location );
 
     my $readlist_orig        = delete $args{read_config};
     my $check                = delete $args{check};
@@ -384,19 +381,25 @@ sub try_read_backend {
         config_file => $config_file_override
     );
 
-    my ( $file_ok, $res, $fh, $file_path );
+    my ( $res, $file_path, $error );
 
     if ( $backend eq 'custom' ) {
+        #$logger->warn("custom read/write backend is deprecated. Please replace with a backend inheriting Config::Model::Backend::Any");
         my $c = my $file = delete $read->{class};
         $file =~ s!::!/!g;
         my $f = delete $read->{function} || 'read';
         require $file . '.pm' unless $c->can($f);
-        no strict 'refs';
 
         $logger->info("Read with custom backend $ {c}::$f in dir $read_dir");
+        my ($file_ok, $fh);
+        ( $file_ok, $file_path ) = $self->get_cfg_file_path(@read_args);
+
+        if ($file_ok and not ($c->can('skip_open') and $c->skip_open)) {
+            $fh = $self->open_read_file($backend, $file_path);
+        }
 
-        ( $file_ok, $file_path, $fh ) = $self->open_read_file(@read_args);
         eval {
+            no strict 'refs';
             $res = &{ $c . '::' . $f }(
                 @read_args,
                 file_path => $file_path,
@@ -404,15 +407,21 @@ sub try_read_backend {
                 object    => $self->node
             );
         };
+        $error = $@;
     }
     elsif ( $backend eq 'perl_file' ) {
-        ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => '.pl' );
-        return ( 0, $file_path ) unless $file_ok;
+        my ($file_ok, $fh);
+        ( $file_ok, $file_path ) = $self->get_cfg_file_path(@read_args, suffix => '.pl' );
+        return ( 0, $file_path ) if not $file_ok or not -r $file_path;
+        $fh = $self->open_read_file($backend, $file_path);
         eval { $res = $self->read_perl( @read_args, file_path => $file_path, io_handle => $fh ); };
+        $error = $@;
     }
     elsif ( $backend eq 'cds_file' ) {
-        ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => '.cds' );
-        return ( 0, $file_path ) unless $file_ok;
+        my ($file_ok, $fh);
+        ( $file_ok, $file_path ) = $self->get_cfg_file_path(@read_args, suffix => '.cds' );
+        return ( 0, $file_path ) if not $file_ok or not -r $file_path;
+        $fh = $self->open_read_file($backend, $file_path);
         eval {
             $res = $self->read_cds_file(
                 @read_args,
@@ -420,6 +429,7 @@ sub try_read_backend {
                 io_handle => $fh,
             );
         };
+        $error = $@;
     }
     else {
         # try to load a specific Backend class
@@ -430,9 +440,16 @@ sub try_read_backend {
         no strict 'refs';
         my $backend_obj = $c->new( node => $self->node, name => $backend );
         $self->set_backend( $backend => $backend_obj );
-        my $suffix;
+        my ($file_ok, $fh, $suffix);
         $suffix = $backend_obj->suffix if $backend_obj->can('suffix');
-        ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => $suffix );
+        ( $file_ok, $file_path ) = $self->get_cfg_file_path(
+            @read_args,
+            suffix => $suffix,
+            skip_compute => $c->skip_open,
+        );
+        $fh = $self->open_read_file($backend, $file_path)
+            if $file_ok and not $c->skip_open;
+
         if ($logger->is_info) {
             my $fp = defined $file_path ? " on $file_path":'' ;
             $logger->info( "Read with $backend " . $c . "::$f".$fp);
@@ -446,6 +463,7 @@ sub try_read_backend {
                 object    => $self->node,
             );
         };
+        $error = $@;
 
         # only backend based on C::M::Backend::Any can support annotations
         if ($backend_obj->can('annotation')) {
@@ -455,17 +473,19 @@ sub try_read_backend {
     }
 
     # catch eval errors done in the if-then-else block before
-    my $e = $@;
-    if ( ref($e) and $e->isa('Config::Model::Exception::Syntax') ) {
+    if ( ref($error) and $error->isa('Config::Model::Exception::Syntax') ) {
 
-        $e->parsed_file( $file_path) unless $e->parsed_file;
-        $e->rethrow;
+        $error->parsed_file( $file_path) unless $error->parsed_file;
+        $error->rethrow;
     }
-    elsif ( ref $e ) {
-        $e->rethrow ;
+    elsif ( ref $error and $error->isa('Config::Model::Exception') ) {
+        $error->rethrow ;
     }
-    elsif ( $e ) {
-        die "Backend error: $e";
+    elsif ( ref $error ) {
+        die $error ;
+    }
+    elsif ( $error ) {
+        die "Backend error: $error";
     }
 
     return ( $res, $file_path );
@@ -476,8 +496,6 @@ sub auto_write_init {
     my $wrlist_orig = delete $args{write_config};
     my $w_dir       = delete $args{write_config_dir};
 
-    weaken($self);    # avoid leak: $self is stored in write_back closure
-
     croak "auto_write_init: unexpected args " . join( ' ', keys %args ) . "\n"
         if %args;
 
@@ -515,7 +533,7 @@ sub auto_write_init {
 
         my $write_dir = $self->get_tuned_config_dir(%$write);
 
-        $logger->debug( "auto_write_init creating write cb ($backend) for ", $self->node->name );
+        $logger->trace( "auto_write_init creating write cb ($backend) for ", $self->node->name );
 
         my @wr_args = (
             %$write,    # model data
@@ -536,8 +554,8 @@ sub auto_write_init {
             $wb = sub {
                 no strict 'refs';
                 $logger->debug( "write cb ($backend) called for ", $self->node->name );
-                my ( $file_ok, $file_path, $fh ) =
-                    $self->open_file_to_write( $backend, @wr_args, @_ )
+                my ( $file_ok, $file_path, $fh );
+                ( $file_ok, $file_path, $fh ) = $self->open_file_to_write( $backend, @wr_args, @_ )
                     unless ( $c->can('skip_open') and $c->skip_open );
                 my $res;
                 $res = eval {
@@ -552,9 +570,10 @@ sub auto_write_init {
                         @_                          # override from user
                     );
                 };
-                $logger->warn( "write backend $c" . '::' . "$f failed: $@" ) if $@;
-                $self->close_file_to_write( $@, $fh, $file_path );
-                return defined $res ? $res : $@ ? 0 : 1;
+                my $error = $@;
+                $logger->warn( "write backend $c" . '::' . "$f failed: $error" ) if $error;
+                $self->close_file_to_write( $error, $fh, $file_path, $write->{file_mode} );
+                return defined $res ? $res : $error ? 0 : 1;
             };
             $self->{auto_write}{custom} = 1;
         }
@@ -567,9 +586,10 @@ sub auto_write_init {
                 $res = eval {
                     $self->write_perl( @wr_args, io_handle => $fh, file_path => $file_path, @_ );
                 };
-                $self->close_file_to_write( $@, $fh, $file_path );
-                $logger->warn("write backend $backend failed: $@") if $@;
-                return defined $res ? $res : $@ ? 0 : 1;
+                my $error = $@;
+                $logger->warn("write backend $backend failed: $error") if $error;
+                $self->close_file_to_write( $error, $fh, $file_path, $write->{file_mode} );
+                return defined $res ? $res : $error ? 0 : 1;
             };
             $self->{auto_write}{perl_file} = 1;
         }
@@ -587,39 +607,54 @@ sub auto_write_init {
                         @_
                     );
                 };
-                $logger->warn("write backend $backend failed: $@") if $@;
-                $self->close_file_to_write( $@, $fh, $file_path );
-                return defined $res ? $res : $@ ? 0 : 1;
+                my $error = $@;
+                $logger->warn("write backend $backend failed: $error") if $error;
+                $self->close_file_to_write( $error, $fh, $file_path, $write->{file_mode} );
+                return defined $res ? $res : $error ? 0 : 1;
             };
             $self->{auto_write}{cds_file} = 1;
         }
         else {
             my $f = $write->{function} || 'write';
             my $c = load_backend_class( $backend, $f );
+            my $location = $self->node->name;
+            my $node = $self->node; # closure
 
             $wb = sub {
-                no strict 'refs';
-                $logger->debug( "write cb ($backend) called for ", $self->node->name );
+                my %cb_args = @_;
+
+                my $force_delete = delete $cb_args{force_delete} ;
+                $logger->debug( "write cb ($backend) called for $location ", $force_delete ? '' : ' (deleted)' );
                 my $backend_obj = $self->get_backend($backend)
                     || $c->new( node => $self->node, name => $backend );
                 my $suffix = $backend_obj->suffix if $backend_obj->can('suffix');
-                my ( $file_ok, $file_path, $fh ) =
-                    $self->open_file_to_write( $backend, suffix => $suffix, @wr_args, @_ )
-                    unless ( $c->can('skip_open') and $c->skip_open );
+                my ( $file_ok, $file_path, $fh );
+                ( $file_ok, $file_path, $fh ) =
+                    $self->open_file_to_write( $backend, suffix => $suffix, @wr_args, %cb_args )
+                    unless $c->skip_open;
 
                 # override needed for "save as" button
                 my %backend_args = (
                         @wr_args,
                         io_handle => $fh,
                         file_path => $file_path,
-                        object    => $self->node,
-                        @_    # override from user
+                        object    => $node,
+                        %cb_args    # override from user
                     );
-                my $res = eval { $backend_obj->$f( %backend_args ); };
-                $logger->warn( "write backend $backend $c" . '::' . "$f failed: $@" ) if $@;
-                $self->close_file_to_write( $@, $fh, $file_path );
 
-                $self->auto_delete($file_path, \%backend_args) if $write->{auto_delete};
+                my $res;
+                if ($force_delete) {
+                    $backend_obj->delete(%backend_args);
+                }
+                else {
+                    $res = eval { $backend_obj->$f( %backend_args ); };
+                    my $error = $@;
+                    $logger->warn( "write backend $backend $c" . '::' . "$f failed: $error" ) if $error;
+                    $self->close_file_to_write( $error, $fh, $file_path, $write->{file_mode} );
+
+                    $self->auto_delete($file_path, \%backend_args)
+                        if $write->{auto_delete} and not $c->skip_open ;
+                }
 
                 return defined $res ? $res : $@ ? 0 : 1;
             };
@@ -627,16 +662,18 @@ sub auto_write_init {
 
         # FIXME: enhance write back mechanism so that different backend *and* different nodes
         # work as expected
-        $logger->debug( "registering write $backend in node " . $self->node->name );
-        push @{ $self->{write_back} }, [ $backend, $wb ];
-        $instance->register_write_back( $self->node->location );
+        $logger->trace( "registering write $backend in node " . $self->node->name );
+
+        $instance->register_write_back(  $self->node->location, $backend, $wb  );
     }
 }
 
 sub auto_delete {
     my ($self, $file_path, $args) = @_;
 
-    my $perl_data = $self->node->dump_as_data( full_dump => $args->{full_dump} // 0);
+    my $perl_data;
+    $perl_data = $self->node->dump_as_data( full_dump => $args->{full_dump} // 0)
+        if defined $self->node;
 
     my $size = ref($perl_data) eq 'HASH'  ? scalar keys %$perl_data
              : ref($perl_data) eq 'ARRAY' ? scalar @$perl_data
@@ -647,40 +684,6 @@ sub auto_delete {
     }
 }
 
-sub write_back {
-    my $self = shift;
-    my %args = @_;
-
-    my $force_backend = delete $args{backend} || '';
-
-    croak "write_back: no subs registered in node", $self->node->location, ". cannot save data\n"
-        unless @{ $self->{write_back} };
-
-    my @backends = @{ $self->{write_back} };
-    $logger->debug(
-        "write_back called on node '",
-        $self->node->name, "' for ", scalar @backends,
-        " backends"
-    );
-
-    my $dir = $args{config_dir};
-    mkpath( $dir, 0, 0755 ) if $dir and not -d $dir;
-
-    foreach my $wb_info (@backends) {
-        my ( $backend, $wb ) = @$wb_info;
-        if (   not $force_backend
-            or $force_backend eq $backend
-            or $force_backend eq 'all' ) {
-
-            # exit when write is successfull
-            my $res = $wb->(%args);
-            $logger->info( "write_back called with $backend backend, result is ",
-                defined $res ? $res : '<undef>' );
-            last if ( $res and not $force_backend );
-        }
-    }
-    $logger->debug( "write_back on node '", $self->node->name, "' done" );
-}
 
 sub open_file_to_write {
     my ( $self, $backend, %args ) = @_;
@@ -702,13 +705,12 @@ sub open_file_to_write {
         }
     }
     elsif ($file_ok) {
-        if ( $do_backup and -r $file_path ) {
-            copy( $file_path, $file_path . $backup ) or die "Backup copy failed: $!";
+        my $file = path($file_path);
+        if ( $do_backup and $file->is_file ) {
+            $file->copy( $file_path . $backup ) or die "Backup copy failed: $!";
         }
         $logger->debug("$backend backend opened file $file_path to write");
-        my $fh = new IO::File;
-        $fh->open("> $file_path") || die "Cannot open $file_path:$!";
-        $fh->binmode(':utf8');
+        my $fh = $file->openw_utf8;
         return ( $file_ok, $file_path, $fh );
     }
     else {
@@ -717,7 +719,7 @@ sub open_file_to_write {
 }
 
 sub close_file_to_write {
-    my ( $self, $error, $fh, $file_path ) = @_;
+    my ( $self, $error, $fh, $file_path, $file_mode ) = @_;
 
     return unless defined $file_path;
 
@@ -735,6 +737,7 @@ sub close_file_to_write {
     }
 
     $fh->close;
+    path($file_path)->chmod($file_mode) if $file_mode;
 
     # check file size and remove empty files
     unlink($file_path) if -z $file_path and not -l $file_path;
@@ -757,6 +760,7 @@ sub read_cds_file {
     return 1;
 }
 
+# TODO: replace with class based on Config::Model::Backend::Any
 sub write_cds_file {
     my $self      = shift;
     my %args      = @_;
@@ -768,6 +772,7 @@ sub write_cds_file {
     return 1;
 }
 
+# TODO: replace with class based on Config::Model::Backend::Any
 sub read_perl {
     my $self = shift;
     my %args = @_;
@@ -813,7 +818,7 @@ Config::Model::BackendMgr - Load configuration node on demand
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -958,7 +963,7 @@ optional as the file name can be hardcoded in the custom class.
 
 The configuration file name can be specified with C<&index> keyword
 when a backend is associated to a node contained in a hash. For instance,
-with C<file> set to C<index.conf>:
+with C<file> set to C<&index.conf>:
 
  service    # hash element
    foo      # hash index
@@ -966,9 +971,28 @@ with C<file> set to C<index.conf>:
    bar      # hash index
      nodeB  # values of nodeB are  stored in bar.conf
 
+Likewise, the keyword C<&element> can be used to specify the file
+name. For instance, with C<file> set to C<&element-&index.conf>:
+
+ service    # hash element
+   foo      # hash index
+     nodeA  # values of nodeA are stored in service.foo.conf
+   bar      # hash index
+     nodeB  # values of nodeB are  stored in service.bar.conf
+
 Alternatively, C<file> can be set to C<->, in which case, the
 configuration is read from STDIN.
 
+=item file_mode
+
+C<file_mode> parameter can be used to set the mode of the written
+file(s). C<file_mode> value can be in any form supported by
+L<Path::Tiny/chmod>. Example:
+
+  file_mode => 0664,
+  file_mode => '0664',
+  file_mode => 'g+w'
+
 =item os_config_dir
 
 Specify alternate location of a configuration directory depending on the OS
@@ -1051,7 +1075,7 @@ mandatory C<config_dir> parameter. For instance:
    },
 
 When C<file> is not specified, a file name is constructed with
-C<< <config_class_name>.<suffix> >> where suffix is C<pl> or C<cds>.
+C<< <instance_name>.<suffix> >> where suffix is C<pl> or C<cds>.
 
 =head2 Custom backend
 
@@ -1183,27 +1207,6 @@ C<write_config> specification.
 
 =head1 Methods
 
-=head2 write_back ( ... )
-
-Try to run all subroutines registered by L<auto_write_init>
-write the configuration information until one succeeds (returns
-true).
-
-You can specify here a pseudo root directory or another config
-directory to write configuration data back with C<root> and
-C<config_dir> parameters. This overrides the model specifications.
-
-You can force to use a backend by specifying C<< backend => xxx >>.
-For instance, C<< backend => 'perl_file' >> or C<< backend => 'custom' >>.
-
-You can force to use all backend to write the files by specifying
-C<< backend => 'all' >>.
-
-You can force a specific config file to write with
-C<< config_file => 'foo/bar.conf' >>
-
-C<write_back> croaks if no write call-back are known for this node.
-
 =head2 support_annotation
 
 Returns 1 if at least one of the backends support to read and write annotations
@@ -1224,7 +1227,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
