@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::CheckList;
-$Config::Model::CheckList::VERSION = '2.097';
+$Config::Model::CheckList::VERSION = '2.106';
 use Mouse;
 use 5.010;
 
@@ -25,6 +25,7 @@ extends qw/Config::Model::AnyThing/;
 with "Config::Model::Role::WarpMaster";
 with "Config::Model::Role::Grab";
 with "Config::Model::Role::HelpAsText";
+with "Config::Model::Role::ComputeFunction";
 
 my $logger = get_logger("Tree::Element::CheckList");
 
@@ -104,10 +105,10 @@ sub set_properties {
     # cleanup all parameters that are handled by warp
     map( delete $self->{$_}, @allowed_warp_params );
 
-    if ( $logger->is_debug() ) {
+    if ( $logger->is_trace() ) {
         my %h = @_;
         my $keys = join( ',', keys %h );
-        $logger->debug("set_properties called on $self->{element_name} with $keys");
+        $logger->trace("set_properties called on $self->{element_name} with $keys");
     }
 
     # merge data passed to the constructor with data passed to set
@@ -158,7 +159,7 @@ sub setup_choice {
     my $self = shift;
     my @choice = ref $_[0] ? @{ $_[0] } : @_;
 
-    $logger->debug("CheckList $self->{element_name}: setup_choice with @choice");
+    $logger->trace("CheckList $self->{element_name}: setup_choice with @choice");
 
     # store all enum values in a hash. This way, checking
     # whether a value is present in the enum set is easier
@@ -605,7 +606,7 @@ sub store_set { goto &set_checked_list }
 
 sub set_checked_list {
     my $self = shift;
-    $logger->debug("called with @_");
+    $logger->trace("called with @_");
     my %set = map { $_ => 1 } @_;
     my @changed;
 
@@ -746,7 +747,7 @@ Config::Model::CheckList - Handle check list element
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -1179,7 +1180,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
