@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ListId;
-$Config::Model::ListId::VERSION = '2.082';
+$Config::Model::ListId::VERSION = '2.106';
 use 5.10.1;
 use Mouse;
 
@@ -18,6 +18,9 @@ use Log::Log4perl qw(get_logger :levels);
 use Carp;
 extends qw/Config::Model::AnyId/;
 
+with "Config::Model::Role::Grab";
+with "Config::Model::Role::ComputeFunction";
+
 my $logger = get_logger("Tree::Element::Id::List");
 
 has data => (
@@ -68,7 +71,7 @@ sub set_properties {
     # warp mechanism
     foreach my $k ( 0 .. $#{$data} ) {
         next unless $k > $self->{max_index};
-        $logger->debug( "set_properties: ", $self->name, " deleting index $k" );
+        $logger->trace( "set_properties: ", $self->name, " deleting index $k" );
         delete $data->[$k];
     }
 }
@@ -132,6 +135,8 @@ sub load {
 
     my @set;
     my $cmd   = $string;
+    $logger->debug( "load: ", $self->name, " called with ->$string<-" );
+
     my $regex = qr/^(
                     (?:
                              "
@@ -182,6 +187,7 @@ sub store_set {
         %args = @_;    # note that $r was shifted out of @_
     }
 
+    $logger->debug(  $self->name, " called with <".join('><',@v).'>' );
     my @comments = @{ $args{comment} || [] };
 
     my $idx = 0;
@@ -499,7 +505,7 @@ Config::Model::ListId - Handle list element for configuration model
 
 =head1 VERSION
 
-version 2.082
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -532,7 +538,7 @@ Returns the number of elements of the list.
 =head2 load(string, [ check => 'no' ] )
 
 Store a set of values passed as a comma separated list of values. 
-Values can be quoted strings. (i.e C<"a,a",b> will yield
+Values can be quoted strings. (i.e C<"a,a",b> yields
 C<('a,a', 'b')> list). 
 
 C<check> can be yes, no or skip
@@ -590,7 +596,7 @@ unshift some values before value equal to C<val> or before value matching C<stuf
 =head2 insort( value1, [ value2 ... ] )
 
 Insert C<zz> value on C<xxx> list so that existing alphanumeric order is preserved.
-Will yield unpexpected results if call on an unsorted list.
+C<insort> yields unpexpected results when called on an unsorted list.
 
 =head2 store
 
@@ -634,14 +640,14 @@ Remove an element from the list. Equivalent to C<splice @list,$idx,1>
 
 Clear and load list from data contained in the C<data> array ref. If a scalar or a hash ref
 is passed, the list is cleared and the data is stored in
-the first element of the list. If split_reg is specified, the scalar will be split
+the first element of the list. If split_reg is specified, the scalar is split
 to load the array.
 
 For instance
 
    $elt->load_data( data => 'foo,bar', split_reg => qr(,) ) ;
 
-will load C< [ 'foo','bar']> in C<$elt>
+loads C< [ 'foo','bar']> in C<$elt>
 
 =head1 AUTHOR
 
@@ -661,7 +667,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
