@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::SimpleUI;
-$Config::Model::SimpleUI::VERSION = '2.097';
+$Config::Model::SimpleUI::VERSION = '2.106';
 use Carp;
 use 5.010;
 use strict;
@@ -133,10 +133,8 @@ my %run_dispatch = (
     help => sub { return $syntax; },
     set  => sub {
         my $self = shift;
-        my $cmd  = shift;
-        if ($cmd) {
-            $cmd =~ s/\s*([=:])\s*/$1/;
-            $self->{current_node}->load($cmd);
+        if (@_) {
+            $self->{current_node}->load(join('',@_));
         }
         else {
             say "No command given.";
@@ -340,7 +338,7 @@ Config::Model::SimpleUI - Simple interface for Config::Model
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -565,7 +563,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
