@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::TermUI;
-$Config::Model::TermUI::VERSION = '2.097';
+$Config::Model::TermUI::VERSION = '2.106';
 use Carp;
 use utf8;      # so literals and identifiers can be in UTF-8
 use v5.12;     # or later to get "unicode_strings" feature
@@ -174,11 +174,16 @@ sub new {
     }
     elsif ( $term->ReadLine eq "Term::ReadLine::Perl" ) {
         no warnings "once";
+        warn "utf-8 support has not beed tested with Term::ReadLine::Perl. ",
+            "You should install Term::ReadLine::Gnu.\n";
         $readline::rl_completion_function = $sub_ref;
         &readline::rl_set( rl_completer_word_break_characters => $word_break_string );
 
         # &readline::rl_set('TcshCompleteMode', 'On');
     }
+    else {
+        warn "You should install Term::ReadLine::Gnu for autocompletion and utf-8 support.\n";
+    }
 
     $self->{term} = $term;
 
@@ -233,7 +238,7 @@ Config::Model::TermUI - Interactive command line interface for cme
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -406,7 +411,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
