@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::TreeSearcher;
-$Config::Model::TreeSearcher::VERSION = '2.082';
+$Config::Model::TreeSearcher::VERSION = '2.106';
 use Mouse;
 use Mouse::Util::TypeConstraints;
 
@@ -56,7 +56,7 @@ sub search {
     my $self   = shift;
     my $string = shift;    # string to search, can be a regexp
 
-    $logger->debug( "TreeSearcher: creating scanner for " . $self->node->name );
+    $logger->trace( "TreeSearcher: creating scanner for " . $self->node->name );
     my $reg = qr/$string/i;
 
     my @scanner_args;
@@ -104,7 +104,7 @@ sub search {
         foreach my $e (@element) {
             my $store = 0;
 
-            map { $store = 1 if $need_search->{$_} and $node->get_help( $_ => $e ) =~ $reg }
+            map { $store = 1 if $need_search->{$_} and $node->get_help_as_text( $_ => $e ) =~ $reg }
                 qw/description summary/;
             $store = 1 if $need_search->{element} and $e =~ $reg;
 
@@ -121,7 +121,7 @@ sub search {
     my $store_sub = sub {
         my $p = shift;
         return if @loc and $loc[$#loc] eq $p;
-        $logger->debug("TreeSearcher: storing location '$p'");
+        $logger->trace("TreeSearcher: storing location '$p'");
         push @loc, $p;
     };
     $scan->scan_node( $store_sub, $self->node );
@@ -147,7 +147,7 @@ Config::Model::TreeSearcher - Search tree for match in value, description...
 
 =head1 VERSION
 
-version 2.082
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -178,8 +178,8 @@ version 2.082
 
  my $root = $inst->config_root ;
 
- my $step = 'baz:fr=bonjour baz:hr="dobar dan" foo="journalled"';
- $root->load( step => $step ) ;
+ my $steps = 'baz:fr=bonjour baz:hr="dobar dan" foo="journalled"';
+ $root->load( steps => $steps ) ;
 
  my @result = $root->tree_searcher(type => 'value')->search('jour');
  print join("\n",@result),"\n" ;
@@ -190,7 +190,7 @@ version 2.082
 =head1 DESCRIPTION
 
 This class provides a way to search the content of a configuration tree. 
-Given a keyword or a pattern, the search method will scan the tree to find
+Given a keyword or a pattern, the search method scans the tree to find
 a value, a description or anything that match the given pattern (or keyword).
 
 =head1 Constructor
@@ -225,7 +225,7 @@ Search in all the items above
 
 Search the keyword or pattern in the tree. The search is done in a case
 insensitive manner. Returns a list of path pointing 
-to the matching tree element. See L<Config::Model::AnyThing/grab(...)> for details
+to the matching tree element. See L<Config::Model::Role::Grab/grab> for details
 on the path syntax.
 
 =head1 BUGS
@@ -249,7 +249,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
