@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Value;
-$Config::Model::Value::VERSION = '2.097';
+$Config::Model::Value::VERSION = '2.106';
 use 5.10.1;
 
 use Mouse;
@@ -34,6 +34,7 @@ extends qw/Config::Model::AnyThing/;
 with "Config::Model::Role::WarpMaster";
 with "Config::Model::Role::Grab";
 with "Config::Model::Role::HelpAsText";
+with "Config::Model::Role::ComputeFunction";
 
 my $logger        = get_logger("Tree::Element::Value");
 my $change_logger = get_logger("Anything::Change");
@@ -178,9 +179,9 @@ sub notify_change {
 
     return if $self->instance->initial_load and not $args{really};
 
-    $change_logger->debug( "called while needs_check is ",
+    $change_logger->trace( "called while needs_check is ",
         $self->needs_check, " for ", $self->name, " with ", join( ' ', %args ) )
-        if $change_logger->is_debug;
+        if $change_logger->is_trace;
     $self->needs_check(1) unless $check_done;
     {
         no warnings 'uninitialized';
@@ -240,7 +241,7 @@ sub set_default {
 sub _build_compute_obj {
     my $self = shift;
 
-    $logger->debug("called");
+    $logger->trace("called");
 
     my $c_info = $self->compute;
     return unless $c_info;
@@ -260,7 +261,7 @@ sub _build_compute_obj {
     my $v = $ret->compute_variables;
 
     $self->register_in_other_value($v);
-    $logger->debug("done");
+    $logger->trace("done");
     return $ret;
 }
 
@@ -286,7 +287,7 @@ sub register_in_other_value {
 # internal
 sub perform_compute {
     my $self = shift;
-    $logger->debug("called");
+    $logger->trace("called");
 
     my $result = $self->compute_obj->compute;
 
@@ -303,7 +304,7 @@ sub perform_compute {
         );
     }
 
-    $logger->debug("done");
+    $logger->trace("done");
     return $ok ? $result : undef;
 }
 
@@ -518,8 +519,8 @@ sub set_properties {
     map { delete $args{$_} } qw/level/;
 
     my $logger = $logger;
-    if ( $logger->is_debug ) {
-        $logger->debug( "Leaf '" . $self->name . "' set_properties called with '",
+    if ( $logger->is_trace ) {
+        $logger->trace( "Leaf '" . $self->name . "' set_properties called with '",
             join( "','", sort keys %args ), "'" );
     }
 
@@ -914,7 +915,7 @@ sub check_value {
 
     $args{value} = $value;    # may be updated by apply_fix
 
-    $logger->debug("done");
+    $logger->trace("done");
 
     $cb->( %args, ok => not @error ) if $cb;
     my $ok = not @error;
@@ -942,7 +943,7 @@ sub run_code_on_value {
         $msg .= " (this cannot be fixed with 'cme fix' command)" unless $fix;
         push @$array, $msg unless $apply_fix;
         $self->{nb_of_fixes}++ if ( defined $fix and not $apply_fix );
-        $self->apply_fix( $fix, $value_r ) if ( defined $fix and $apply_fix );
+        $self->apply_fix( $fix, $value_r, $msg ) if ( defined $fix and $apply_fix );
     }
 }
 
@@ -995,8 +996,8 @@ sub has_fixes {
 sub apply_fixes {
     my $self = shift;
 
-    if ( $logger->is_debug ) {
-        $fix_logger->debug( "called for " . $self->location );
+    if ( $logger->is_trace ) {
+        $fix_logger->trace( "called for " . $self->location );
     }
 
     my ( $old, $new );
@@ -1019,11 +1020,11 @@ sub apply_fixes {
 
 # internal: called by check when a fix is required
 sub apply_fix {
-    my ( $self, $fix, $value_r ) = @_;
+    my ( $self, $fix, $value_r, $msg ) = @_;
 
     local $_ = $$value_r;    # used inside $fix sub ref
 
-    if ( $fix_logger->is_debug ) {
+    if ( $fix_logger->is_info ) {
         my $str = $fix;
         $str =~ s/\n/ /g;
         $fix_logger->info( $self->location . ": Applying fix '$str'" );
@@ -1040,7 +1041,7 @@ sub apply_fix {
     no warnings "uninitialized";
     if ( $_ ne $$value_r ) {
         $fix_logger->info( $self->location . ": fix changed value from '$$value_r' to '$_'" );
-        $self->_store_fix( $$value_r, $_ );
+        $self->_store_fix( $$value_r, $_, $msg );
     }
     else {
         $fix_logger->info( $self->location . ": fix did not change value '$$value_r'" );
@@ -1048,7 +1049,7 @@ sub apply_fix {
 }
 
 sub _store_fix {
-    my ( $self, $old, $new ) = @_;
+    my ( $self, $old, $new, $msg ) = @_;
 
     $self->{data} = $new;
 
@@ -1060,7 +1061,7 @@ sub _store_fix {
     my %args = (
         old => $old // $self->_fetch_std,
         new => $new // $self->_fetch_std,
-        note => 'applied fix'
+        note => 'applied fix'. ( $msg ? ' for :'. $msg : ''),
     ) ;
     no warnings "uninitialized";
     # in case $old is the default value and $new is undef
@@ -1164,7 +1165,7 @@ sub store {
 
     if ( defined $old_value and $value eq $old_value ) {
         $logger->info( "skip storage of ", $self->composite_name, " unchanged value: $value" )
-            if $logger->is_debug;
+            if $logger->is_info;
         return 1;
     }
 
@@ -1267,7 +1268,7 @@ sub _store {
         $self->trigger_warp($value);
     }
 
-    $logger->debug( "_store done on ", $self->composite_name ) if $logger->is_debug;
+    $logger->trace( "_store done on ", $self->composite_name ) if $logger->is_trace;
 }
 
 #
@@ -1289,7 +1290,7 @@ sub transform_boolean {
 
     # convert yes no to 1 or 0
     $$v_ref = 1 if ( $$v_ref =~ /^y/i or $$v_ref =~ /true/i );
-    $$v_ref = 0 if ( $$v_ref =~ /^n/i or $$v_ref =~ /false/i );
+    $$v_ref = 0 if ( $$v_ref =~ /^n/i or $$v_ref =~ /false/i or length($$v_ref) == 0);
 }
 
 # internal. return ( undef, value)
@@ -1514,7 +1515,7 @@ my %accept_mode = map { ( $_ => 1 ) } qw/custom standard preset default upstream
 
 sub _fetch {
     my ( $self, $mode, $check ) = @_;
-    $logger->debug( "called for " . $self->location ) if $logger->is_debug;
+    $logger->trace( "called for " . $self->location ) if $logger->is_trace;
 
     # always call to perform submit_to_warp
     my $pref = $self->_fetch_std( $check );
@@ -1528,8 +1529,9 @@ sub _fetch {
         $self->notify_change( old => undef, new => $pref, note => "use $info value" );
     }
 
+    my $layer_data = $self->{layered};
     my $known_upstream =
-          defined $self->{layered}           ? $self->{layered}
+          defined $layer_data                ? $layer_data
         : $self->compute_is_upstream_default ? $self->perform_compute
         :                                      $self->{upstream_default};
     my $std = defined $pref ? $pref : $known_upstream;
@@ -1556,7 +1558,7 @@ sub _fetch {
         $cust = $data
             if $data ne $pref
             and $data ne $self->{upstream_default}
-            and $data ne $self->{layered};
+            and $data ne $layer_data;
         $logger->debug( "custom mode result '$cust' for " . $self->location )
             if $logger->is_debug;
         return $cust;
@@ -1564,10 +1566,13 @@ sub _fetch {
 
     if ( $mode eq 'non_upstream_default' ) {
         no warnings "uninitialized";
-        my $nbu =
-              defined $data && $data ne $self->{upstream_default} ? $data
-            : defined $pref && $pref ne $self->{upstream_default} ? $pref
-            :                                                       undef;
+        my $nbu;
+        foreach my $d ($data, $layer_data, $pref) {
+            if ( defined $d and $d ne $self->{upstream_default} ) {
+                $nbu = $d;
+                last;
+            }
+        }
 
         $logger->debug( "done in non_upstream_default mode for " . $self->location )
             if $logger->is_debug;
@@ -1622,8 +1627,8 @@ sub fetch {
     my $silent = $args{silent} || 0;
     my $check  = $self->_check_check( $args{check} );
 
-    if ( $logger->is_debug ) {
-        $logger->debug( "called for "
+    if ( $logger->is_trace ) {
+        $logger->trace( "called for "
                 . $self->location
                 . " check $check mode $mode"
                 . " needs_check "
@@ -1663,8 +1668,8 @@ sub fetch {
     $ok = $self->check( value => $value, silent => $silent, mode => $mode )
         if $mode =~ /backend|custom|user/;
 
-    $logger->debug( "$mode fetch (almost) done for " . $self->location )
-        if $logger->is_debug;
+    $logger->trace( "$mode fetch (almost) done for " . $self->location )
+        if $logger->is_trace;
 
     # check validity (all modes)
     if ( $ok or $check eq 'no' ) {
@@ -1807,7 +1812,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -2182,8 +2187,8 @@ This modules can check several value types:
 
 =item C<boolean>
 
-Accepts values C<1> or C<0>, C<yes> or C<no>, C<true> or C<false>. The
-value read back is always C<1> or C<0>.
+Accepts values C<1> or C<0>, C<yes> or C<no>, C<true> or C<false>, and
+empty string. The value read back is always C<1> or C<0>.
 
 =item C<enum>
 
@@ -2840,7 +2845,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
