@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ValueComputer;
-$Config::Model::ValueComputer::VERSION = '2.097';
+$Config::Model::ValueComputer::VERSION = '2.106';
 use Mouse;
 use MouseX::StrictConstructor;
 
@@ -28,7 +28,13 @@ has formula    => ( is => 'ro', isa => 'Str', required => 1 );
 has value_type => ( is => 'ro', isa => 'Str', required => 1 );
 
 # value_object is mostly used for error messages
-has value_object => ( is => 'ro', isa => 'Config::Model::AnyThing', required => 1, weak_ref => 1 );
+has value_object => (
+    is => 'ro',
+    isa => 'Config::Model::AnyThing',
+    required => 1,
+    weak_ref => 1,
+    handles => [qw/grab grab_value location index element/],
+);
 
 has variables => ( is => 'ro', isa => 'HashRef', default => sub { {} } );
 has replace   => ( is => 'ro', isa => 'HashRef', default => sub { {} } );
@@ -82,15 +88,15 @@ sub BUILD {
     # create parser if needed
     $compute_parser ||= Parse::RecDescent->new($compute_grammar);
 
-    $logger->debug("called with formula: $self->{formula}");
+    $logger->trace("called with formula: $self->{formula}");
     # must make a first pass at computation to subsitute index and
     # element values.  leaves $xxx outside of &index or &element untouched
     my $result_r =
         $compute_parser->pre_compute( $self->{formula}, 1, $self->{value_object},
         $self->{variables}, $self->{replace}, 'yes', $self->need_quote, );
 
-    $logger->debug("pre_formula: $$result_r");
-    $self->{pre_formula} = $$result_r;
+    $logger->trace("pre_formula: ". ($result_r ? $$result_r : ' pre_compute failed, using original formula'));
+    $self->{pre_formula} = $result_r ? $$result_r : $self->{formula};
 }
 
 sub compute {
@@ -99,7 +105,7 @@ sub compute {
     my $check = $args{check} || 'yes';
 
     my $pre_formula = $self->{pre_formula};
-    $logger->debug("called with pre_formula: $pre_formula");
+    $logger->trace("called with pre_formula: $pre_formula");
     my $variables = $self->compute_variables( check => $check );
 
     die "internal error" unless defined $variables;
@@ -112,7 +118,7 @@ sub compute {
 
     if (   $self->{use_eval}
         or $self->{value_type} =~ /(integer|number|boolean)/ ) {
-        $logger->debug("will use eval");
+        $logger->trace("will use eval");
         my $all_defined = 1;
         my @init;
         foreach my $key ( sort keys %$variables ) {
@@ -142,7 +148,7 @@ sub compute {
         }
     }
     else {
-        $logger->debug("calling parser with compute on pre_formula $pre_formula");
+        $logger->trace("calling parser with compute on pre_formula $pre_formula");
         my $formula_r = $compute_parser->compute( $pre_formula, 1, @parser_args );
 
         $result = $$formula_r;
@@ -159,7 +165,7 @@ sub compute_info {
     my $self  = shift;
     my %args  = @_;
     my $check = $args{check} || 'yes';
-    $logger->debug("compute_info called with $self->{formula}");
+    $logger->trace("compute_info called with $self->{formula}");
 
     my $orig_variables = $self->{variables};
     my $variables      = $self->compute_variables;
@@ -217,8 +223,8 @@ sub compute_variables {
     # a shallow copy should be enough as we don't allow
     # replace in replacement rules
     my %variables = %{ $self->{variables} };
-    $logger->debug( "called on variables '", join( "', '", sort keys %variables ), "'" )
-        if $logger->is_debug;
+    $logger->trace( "called on variables '", join( "', '", sort keys %variables ), "'" )
+        if $logger->is_trace;
 
     # apply a compute on all variables until no $var is left
     my $var_left = scalar( keys %variables ) + 1;
@@ -230,16 +236,16 @@ sub compute_variables {
             next unless defined $value;
 
             #next if ref($value); # skip replacement rules
-            $logger->debug("key '$key', value '$value', left $var_left");
+            $logger->trace("key '$key', value '$value', left $var_left");
             next unless $value =~ /\$|&/;
 
             my $pre_res_r =
                 $compute_parser->pre_compute( $value, 1, $self->{value_object}, \%variables,
                 $self->{replace}, $check );
-            $logger->debug("key '$key', pre res '$$pre_res_r', left $var_left\n");
+            $logger->trace("key '$key', pre res '$$pre_res_r', left $var_left\n");
             $variables{$key} = $$pre_res_r;
-            $logger->debug( "variable after pre_compute: ", join( " ", keys %variables ) )
-                if $logger->is_debug;
+            $logger->trace( "variable after pre_compute: ", join( " ", keys %variables ) )
+                if $logger->is_trace;
 
             if ( $$pre_res_r =~ /\$/ ) {
                 ;
@@ -251,12 +257,12 @@ sub compute_variables {
 
                 #return undef unless defined $res ;
                 $variables{$key} = $$res_ref;
-                $logger->debug( "variable after compute: ", join( " ", keys %variables ) )
-                    if $logger->is_debug;
+                $logger->trace( "variable after compute: ", join( " ", keys %variables ) )
+                    if $logger->is_trace;
             }
             {
                 no warnings "uninitialized";
-                $logger->debug("result $key -> '$variables{$key}' left '$var_left'");
+                $logger->trace("result $key -> '$variables{$key}' left '$var_left'");
             }
         }
 
@@ -271,14 +277,14 @@ sub compute_variables {
         ) unless ( $var_left < $old_var_left );
     }
 
-    $logger->debug("done");
+    $logger->trace("done");
     return \%variables;
 }
 
 sub _pre_replace {
     my ( $replace_h, $pre_value ) = @_;
 
-    $logger->debug("value: _pre_replace called with value '$pre_value'");
+    $logger->trace("value: _pre_replace called with value '$pre_value'");
     my $result =
         exists $replace_h->{$pre_value}
         ? $replace_h->{$pre_value}
@@ -290,9 +296,9 @@ sub _replace {
     my ( $replace_h, $value, $value_object, $variables, $replace, $check, $need_quote, $undef_is )
         = @_;
 
-    if ( $logger->is_debug ) {
+    if ( $logger->is_trace ) {
         my $str = defined $value ? $value : '<undef>';
-        $logger->debug("value: _replace called with value '$str'");
+        $logger->trace("value: _replace called with value '$str'");
     }
 
     my $result;
@@ -312,47 +318,10 @@ sub _function_on_object {
     my ( $up, $function, $return, $value_object, $variables_h, $replace_h, $check, $need_quote ) =
         @_;
 
-    $logger->debug("handling &$function($up) ");
+    $logger->trace("handling &$function($up) ");
 
-    # get now the object refered
-    $up =~ s/-(\d+)/'- ' x $1/e;
-
-    my $target = eval { $value_object->grab( step => $up, check => $check ) };
-
-    if ($@) {
-        my $e = $@;
-        my $msg = $e ? $e->full_message : '';
-        Config::Model::Exception::Model->throw(
-            object => $value_object,
-            error  => "Compute function argument '$up':\n" . $msg
-        );
-    }
-
-    if ( $function eq 'element' ) {
-        my $result = $target->element_name;
-        Config::Model::Exception::Model->throw(
-            object => $value_object,
-            error  => "'",
-            $target->name, "' has no element name"
-        ) unless defined $result;
-        $return = \$result;
-    }
-    elsif ( $function eq 'index' ) {
-        my $result = $target->index_value;
-        Config::Model::Exception::Formula->throw(
-            object => $value_object,
-            error  => "'",
-            $target->name, "' has no index value"
-        ) unless defined $result;
-        $return = \$result;
-    }
-    else {
-        Config::Model::Exception::Formula->throw(
-            object => $value_object,
-            error  => "Unknown computation function &$function, "
-                . "expected &element(...) or &index(...)"
-        );
-    }
+    my $target = $value_object->eval_function($function, $up, $check);
+    $return = \$target ;
 
     # print "\&foo(...) result = ",$$return," \n";
 
@@ -366,7 +335,7 @@ sub _function_on_object {
 sub _function_alone {
     my ( $f_name, $return, $value_object, $variables_h, $replace_h, $check, $need_quote ) = @_;
 
-    $logger->debug("_function_alone: handling $f_name");
+    $logger->trace("_function_alone: handling $f_name");
 
     my $method_name =
           $f_name eq 'element'  ? 'element_name'
@@ -499,7 +468,7 @@ pre_value:
   <skip:''> '$replace' '{' /\s*/ pre_value[@arg] /\s*/ '}' {
     $return = Config::Model::ValueComputer::_pre_replace($arg[2], ${ $item{pre_value} } ) ;
   }
-  | <skip:''> function '(' <commit> /\s*/ up /\s*/ ')' {
+  | <skip:''> function '(' /\s*/ up /\s*/ ')' {
     $return = Config::Model::ValueComputer::_function_on_object($item{up},$item{function},$return,@arg ) ;
   }
   | <skip:''> '&' /\w+/ func_param(?) {
@@ -523,7 +492,7 @@ pre_value:
 
 func_param: /\(\s*\)/
 
-up: /-?\d*/
+up: /-\d+|-( ?-)*/
 
 compute:  <skip:''> value[@arg](s) { 
     # if one value is undef, return undef;
@@ -581,7 +550,7 @@ Config::Model::ValueComputer - Provides configuration value computation
 
 =head1 VERSION
 
-version 2.097
+version 2.106
 
 =head1 SYNOPSIS
 
@@ -592,20 +561,21 @@ version 2.097
  $model ->create_config_class (
     name => "MyClass",
 
-    element => [ 
-
-       [qw/av bv/] => {type => 'leaf',
-                       value_type => 'integer',
-                      },
-       compute_int => { 
-	    type => 'leaf',
-            value_type => 'integer',
-            compute    => { formula   => '$a + $b', 
-                            variables => { a => '- av', b => '- bv'}
-                          },
-          },
-   ],
- ) ;
+    element => [
+       [qw/av bv/] => {
+           type => 'leaf',
+           value_type => 'integer',
+       },
+       compute_int => {
+	       type => 'leaf',
+               value_type => 'integer',
+               compute    => {
+                   formula   => '$a + $b',
+                   variables => { a => '- av', b => '- bv'}
+               },
+       },
+    ],
+ );
 
  my $inst = $model->instance(root_class_name => 'MyClass' );
 
@@ -688,7 +658,7 @@ The index value of the current object : C<&index> or C<&index()>.
 =item *
 
 The index value of a parent object: C<&index(-)>. Ancestor index value can be retrieved
-with C<&index(-2)> or C<&index(-3)>.
+with C<&index(-2)> or C<&index(-3)> or C<&index(- -)> or C<&index(- - -)>
 
 =item *
 
@@ -718,16 +688,18 @@ is used as the computed value.
 For instance:
 
   # extract host from url
-  compute => { formula => '$old =~ m!http://[\w\.]+(?::\d+)?(/.*)!; $1 ;', 
-	       variables => { old => '- url' } ,
-	       use_eval => 1 ,
-	     },
+  compute => {
+      formula => '$old =~ m!http://[\w\.]+(?::\d+)?(/.*)!; $1 ;',
+      variables => { old => '- url' } ,
+	  use_eval => 1 ,
+  },
 
   # capitalize
-  compute => { formula => 'uc($old)',
-	       variables => { old => '- small_caps' } ,
-	       use_eval => 1 
-             }
+  compute => {
+      formula => 'uc($old)',
+	  variables => { old => '- small_caps' } ,
+	  use_eval => 1
+  }
 
 =head2 Compute variables
 
@@ -739,33 +711,36 @@ the correct L<Value|Config::Model::Value> object.
 In this numeric example, C<result> default value is C<av + bv>:
 
  element => [
-  av => { 
-    type => 'leaf',
-    value_type => 'integer'
-  },
-  bv => { 
-    type => 'leaf',
-    value_type => 'integer'
-  },
-  result => { 
-    type => 'leaf',
-    value_type => 'integer', 
-    compute => { formula => '$a + $b' , 
-                 variables => { a => '- av', b => '- bv' },
-               }
-  }
+    av => {
+        type => 'leaf',
+        value_type => 'integer'
+    },
+    bv => {
+        type => 'leaf',
+        value_type => 'integer'
+    },
+    result => {
+        type => 'leaf',
+        value_type => 'integer',
+        compute => {
+            formula => '$a + $b' ,
+            variables => { a => '- av', b => '- bv' },
+        }
+    }
+ ]
 
 In this string example, the default value of the C<Comp> element is
 actually a string made of "C<macro is >" and the value of the
 "C<macro>" element of the object located 2 nodes above:
 
-   comp => { 
+ comp => {
     type => 'leaf',
-    value_type => 'string', 
-    compute => { formula => '"macro is $m"' ,
-                 variables => { m => '- - macro' }
-               }
-   }
+    value_type => 'string',
+    compute => {
+        formula => '"macro is $m"' ,
+        variables => { m => '- - macro' }
+    }
+ }
 
 =head2 Compute replace
 
@@ -776,17 +751,18 @@ C<%replace> hash.
 
 For instance, if you want to display a summary of a config, you can do :
 
-       compute_with_replace 
-       => {
-            formula => '$replace{$who} is the $replace{$what} of $replace{$country}',
-            variables => {
-                           who   => '! who' ,
-                           what  => '! what' ,
-                           country => '- country',
-                         },
-            replace => {  chief => 'president', 
-                          America => 'USA'
-                       },
+ compute_with_replace => {
+     formula => '$replace{$who} is the $replace{$what} of $replace{$country}',
+     variables => {
+         who   => '! who' ,
+         what  => '! what' ,
+         country => '- country',
+     },
+     replace => {
+         chief => 'president',
+         America => 'USA'
+     }
+ }
 
 =head2 Complex formula
 
@@ -801,19 +777,23 @@ I.e. C<&element($foo)> is ok, but C<&element(&index($foo))> is not allowed.
 
 Compute variables can themselves be computed :
 
-   compute => {
-     formula => 'get_element is $replace{$s}, indirect value is \'$v\'',
-     variables => { 's' => '! $where',
-                     where => '! where_is_element',
-                     v => '! $replace{$s}',
-                  }
-     replace   => { m_value_element => 'm_value',
-                    compute_element => 'compute' 
-                  }
+ compute => {
+    formula => 'get_element is $replace{$s}, indirect value is \'$v\'',
+    variables => {
+        s => '! $where',
+        where => '! where_is_element',
+        v => '! $replace{$s}',
     }
+    replace => {
+        m_value_element => 'm_value',
+        compute_element => 'compute'
+    }
+ }
 
 Be sure not to specify a loop when doing recursive computation.
 
+The function C<&index> C<&element> and C<&location> are also allowed.
+
 =head2 compute override
 
 In some case, a computed value must be interpreted as a default value
@@ -821,30 +801,32 @@ and the user must be able to override this computed default value.  In
 this case, you must use C<< allow_override => 1 >> with the
 compute parameter:
 
-   computed_value_with_override => { 
+ computed_value_with_override => {
     type => 'leaf',
-    value_type => 'string', 
-    compute => { formula => '"macro is $m"' , 
-                 variables => { m => '- - macro' } ,
-                 allow_override => 1,
-               }
-   }
+    value_type => 'string',
+    compute => {
+        formula => '"macro is $m"' ,
+        variables => { m => '- - macro' } ,
+        allow_override => 1,
+    }
+ }
 
 This computed default value is written to the configuration file.
 
 This default value may be already known by the application so the computed value
-should B<not> be written to the configuration file. The computed value is interesting 
+should B<not> be written to the configuration file. The computed value is interesting
 because it cab be shown to the user. In this case, use the C<use_as_upstream_default>
-parameter: 
+parameter:
 
-   compute_known_upstream => { 
+ compute_known_upstream => {
     type => 'leaf',
-    value_type => 'string', 
-    compute => { formula => '"macro is $m"' , 
-                 variables => { m => '- - macro' } ,
-                 use_as_upstream_default => 1,
-               }
-   }
+    value_type => 'string',
+    compute => {
+        formula => '"macro is $m"' ,
+        variables => { m => '- - macro' } ,
+        use_as_upstream_default => 1,
+    }
+ }
 
 C<use_as_upstream_default> implies C<allow_override>.
 
@@ -865,80 +847,80 @@ The most useful fallback values are:
 
 Example:
 
-        Source => {
-            value_type   => 'string',
-            mandatory    => 1,
-            migrate_from => {
-                use_eval  => 1,
-                formula   => '$old || $older ;',
-                undef_is => "''",
-                variables => {
-                    older => '- Original-Source-Location',
-                    old   => '- Upstream-Source'
-                }
-            },
-            type => 'leaf',
-        },
-        [qw/Upstream-Source Original-Source-Location/] => {
-            value_type => 'string',
-            status     => 'deprecated',
-            type       => 'leaf'
+ Source => {
+    type => 'leaf',
+    value_type   => 'string',
+    mandatory    => 1,
+    migrate_from => {
+        use_eval  => 1,
+        formula   => '$old || $older ;',
+        undef_is => "''",
+        variables => {
+            older => '- Original-Source-Location',
+            old   => '- Upstream-Source'
         }
+    },
+ },
+ [qw/Upstream-Source Original-Source-Location/] => {
+    value_type => 'string',
+    status     => 'deprecated',
+    type       => 'leaf'
+ }
 
 =head1 Examples
 
 =head2 String substitution
 
-    [qw/sav sbv/] => {
-        type       => 'leaf',
-        value_type => 'string',
-      },
-    compute_string => {
-        type       => 'leaf',
-        value_type => 'string',
-        compute    => {
-            formula   => 'meet $a and $b',
-            variables => { '- sav', b => '- sbv' }
-        },
-    },
+ [qw/sav sbv/] => {
+     type       => 'leaf',
+     value_type => 'string',
+   },
+ compute_string => {
+     type       => 'leaf',
+     value_type => 'string',
+     compute    => {
+         formula   => 'meet $a and $b',
+         variables => { '- sav', b => '- sbv' }
+     },
+ },
 
 =head2 Computation with on-the-fly replacement
 
-    compute_with_replace => {
-        type       => 'leaf',
-        value_type => 'string',
-        compute    => {
-            formula =>
-              '$replace{$who} is the $replace{$what} of $replace{$country}',
-            variables => {
-                who     => '! who',
-                what    => '! what',
-                country => '- country',
-            },
-            replace => {
-                chief   => 'president',
-                America => 'USA'
-            },
-        },
-      },
+ compute_with_replace => {
+     type       => 'leaf',
+     value_type => 'string',
+     compute    => {
+         formula =>
+           '$replace{$who} is the $replace{$what} of $replace{$country}',
+         variables => {
+             who     => '! who',
+             what    => '! what',
+             country => '- country',
+         },
+         replace => {
+             chief   => 'president',
+             America => 'USA'
+         },
+     },
+   },
 
 =head2 Extract data from a value using a Perl regexp
 
 Extract the host name from an URL:
 
-    url => {
-        type       => 'leaf',
-        value_type => 'uniline'
-    },
-    extract_host_from_url => {
-        type       => 'leaf',
-        value_type => 'uniline',
-        compute    => {
-            formula   => '$old =~ m!http://([\w\.]+)!; $1 ;',
-            variables => { old => '- url' },
-            use_eval  => 1,
-        },
-    },
+ url => {
+     type       => 'leaf',
+     value_type => 'uniline'
+ },
+ extract_host_from_url => {
+     type       => 'leaf',
+     value_type => 'uniline',
+     compute    => {
+         formula   => '$old =~ m!http://([\w\.]+)!; $1 ;',
+         variables => { old => '- url' },
+         use_eval  => 1,
+     },
+ },
 
 =head2 simple copy hash example
 
@@ -1033,7 +1015,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
