@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,29 +9,29 @@
 #
 [
   {
-    'name' => 'Fstab::Iso9660_Opt',
-    'include' => [
-      'Fstab::CommonOptions'
+    'author' => [
+      'Dominique Dumont'
     ],
     'copyright' => [
       '2010,2011 Dominique Dumont'
     ],
-    'author' => [
-      'Dominique Dumont'
-    ],
-    'license' => 'LGPL2',
     'element' => [
       'rock',
       {
-        'value_type' => 'boolean',
-        'type' => 'leaf'
+        'type' => 'leaf',
+        'value_type' => 'boolean'
       },
       'joliet',
       {
-        'value_type' => 'boolean',
-        'type' => 'leaf'
+        'type' => 'leaf',
+        'value_type' => 'boolean'
       }
-    ]
+    ],
+    'include' => [
+      'Fstab::CommonOptions'
+    ],
+    'license' => 'LGPL2',
+    'name' => 'Fstab::Iso9660_Opt'
   }
 ]
 ;
